/*
     This file is part of GNUnet.
     (C) 2001, 2002 Christian Grothoff (and other contributing authors)

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * @file include/applications/afs/encoding/searchutil.h
 * @author Christian Grothoff
 *
 * Helper functions for searching.
 **/

#ifndef SEARCHUTIL_H
#define SEARCHUTIL_H

#include "config.h"

#include "applications/afs/protocol/messages.h"

/**
 * Type of a callback method for results that have
 * been received.
 *
 * @param root the RootNode of the result that has been received
 * @param data the opaque handle (context for the callee)
 **/
typedef void (*SearchResultCallback)(RootNode * root,
				     void * data);

/**
 * Method to test if the receive-thread should
 * terminate.
 **/
typedef int (*TestTerminateThread)(void * context);

/**
 * Start retrieving results from GNUnet. This method terminates only
 * if the testTerminate-method returns YES after a result was received
 * or there is an error with reading from the socket.
 *
 * @param sock socket used to receive results from
 * @param keyCount the number of keywords
 * @param keywords the keywords (for decryption)
 * @param messages the queries (to match against)
 * @param handler the method to call on each result matching all keywords
 * @param handlerArgs the arguments to the result handler method
 * @param testTerminate method used to check if we should termiante
 * @param ttContext argument for testTerminate
 **/
void receiveResults(GNUNET_TCP_SOCKET * sock,
  		    int keyCount,
		    HashCode160 * keywords,
		    AFS_CS_QUERY ** messages,
		    SearchResultCallback handler,
		    void * handlerArgs,
		    TestTerminateThread testTerminate,
		    void * ttContext);

/**
 * Context of the sendQueries cron-job.
 **/
typedef struct {
  /**
   * Time when the cron-job was first started.
   **/
  cron_t start;

  /**
   * How many cron-units may we run (total)?
   **/
  cron_t timeout;

  /**
   * Socket for communication with gnunetd
   **/
  GNUNET_TCP_SOCKET * sock;

  /**
   * Number of queries.
   **/
  int queryCount;

  /**
   * queryCount query messages
   **/
  AFS_CS_QUERY ** messages;
} SendQueriesContext;

/**
 * Repeatedly send out the queries to GNUnet.
 * This cron-job self-terminates after sqc.timout
 * cron-units have passed.
 *
 * @param sqc the context
 **/
void sendQueries(SendQueriesContext * sqc);

/**
 * Build an initial set of query messages
 * from the list of keywords.
 * @param keyCount the number of keywords
 * @param keywords the keywords (or keys)
 * @param messages the resulting query messages
 **/
void buildMessages(int keyCount,
		   HashCode160 * keywords,
		   AFS_CS_QUERY *** messages);

/**
 * Parse the keywords (join at spaces, separate at AND).
 * @param num_keywords the number of ascii-keywords
 * @param keywords the list of ascii-keywords
 * @param keys the hashs of the keywords to set (= the keys, not the queries!)
 * @return -1 on error, 0 if we should exit without error,
 *      number of keys if we actually are going to do something
 **/
int parseKeywords(int num_keywords,
		  char ** keywords,
		  HashCode160 ** keys);

#endif
/* end of searchutil.h */
